module CKAdaptee # :nodoc:
class CGI
	attr_accessor :headers, :params
	attr_reader :input, :output, :error

	def initialize
		@headers = ENV
		@input   = $stdin
		@output  = $stdout
		@error   = $stderr
		_initialize_query
	end

	def request
		CKRequest.new( headers, params )
	end

	def response
		CKResponse.new headers
	end

	def out( response )
		puts response
	end

	protected

	# Copied from cgi.rb.
	# Other interfaces should override _parse* methods.
    def _initialize_query
		if ("POST" == @headers['REQUEST_METHOD']) and
			 %r|\Amultipart/form-data.*boundary=\"?([^\";,]+)\"?|n.match(@headers['CONTENT_TYPE'])
			_parse_multipart
		else
			case @headers['REQUEST_METHOD']
			when "HEAD" then query = _parse_head
			when "GET"  then query = _parse_get
			when "POST" then query = _parse_post
			else             query = _parse_else end
			@params = CKUtilities.parse_query_string query
		end
	end

	def _parse_multi_part
		boundary = $1.dup
		@params = read_multipart(boundary, Integer(@headers['CONTENT_LENGTH']))
	end

	def _parse_get
		@headers['QUERY_STRING'] or ""
	end

	alias :_parse_head :_parse_get

	def _parse_post
		@input.binmode
		@input.read(Integer(@headers['CONTENT_LENGTH'])) or ''
	end

	def _parse_else
		require "shellwords"

		string = unless ARGV.empty?
			ARGV.join(' ')
		else
			if STDIN.tty?
				STDERR.print(
					%|(offline mode: enter name=value pairs on standard input)\n|
				)
			end
			readlines.join(' ').gsub(/\n/n, '')
		end.gsub(/\\=/n, '%3D').gsub(/\\&/n, '%26')

		words = Shellwords.shellwords(string)

		if words.find{|x| /=/n.match(x) }
			words.join('&')
		else
			words.join('+')
		end
	end
end
end

