# CKUtilitis is a class wihch collects utility methods based on cgi.rb.
class CKUtilities
	CR            = "\015"
	LF            = "\012"
	EOL           = CR + LF
	RFC822_DAYS   = %w[ Sun Mon Tue Wed Thu Fri Sat ]
	RFC822_MONTHS = %w[ Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec ]

	class << self
		# Returns an encoded string for URL.
		def escape_url( string )
			string.to_s.gsub( /([^ a-zA-Z0-9_.-]+)/n ) do
				'%' + $1.unpack( 'H2' * $1.size ).join( '%' ).upcase
			end.tr( ' ', '+' )
		end

		# Returns a string decoded from URL.
		def unescape_url( string )
			string.to_s.tr( '+', ' ' ).gsub( /((?:%[0-9a-fA-F]{2})+)/n ) do
				[ $1.delete( '%' ) ].pack( 'H*' )
			end
		end

		# Escapes HTML control characters.
		def escape_html( string )
			string.to_s.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').
				gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
		end

		# Unescapes HTML control characters.
		def unescape_html( string )
    		string.to_s.gsub(/&(.*?);/n) do
		    	match = $1.dup
		    	case match
	    		when /\Aamp\z/ni           then '&'
				when /\Aquot\z/ni          then '"'
				when /\Agt\z/ni            then '>'
				when /\Alt\z/ni            then '<'
				when /\A#0*(\d+)\z/n       then
					if Integer($1) < 256
						Integer($1).chr
					else
						if Integer($1) < 65536 and \
							($KCODE[0] == ?u or $KCODE[0] == ?U)
							[Integer($1)].pack("U")
						else
							"&##{$1};"
						end
					end
				when /\A#x([0-9a-f]+)\z/ni then
					if $1.hex < 256
						$1.hex.chr
					else
						if $1.hex < 65536 and \
							($KCODE[0] == ?u or $KCODE[0] == ?U)
							[$1.hex].pack("U")
						else
							"&#x#{$1};"
						end
					end
				else
					"&#{match};"
				end
			end
		end

		# Formats Time object in RFC1123.
		# For example, "Sat, 1 Jan 2000 00:00:00 GMT".
		def date( time )
			t = time.clone.gmtime
			return format("%s, %.2d %s %.4d %.2d:%.2d:%.2d GMT",
				RFC822_DAYS[t.wday], t.day, RFC822_MONTHS[t.month-1], t.year,
				t.hour, t.min, t.sec)
		end

		def parse_query_string( query )
			params = Hash.new([])
			query.split(/[&;]/n).each do |pairs|
				key, value = pairs.split('=',2).collect{|v| unescape_url(v) }
				if params.has_key?(key)
					params[key].push(value)
				else
					params[key] = [value]
				end
			end
			params
		end

 		def parse_cookie( raw_cookie )
			cookies = []
			return cookies unless raw_cookie

			raw_cookie.split('; ').each do |pairs|
				name, value = pairs.split('=',2)
				name  = unescape_url name
				value = unescape_url value

				cookies << CKCookie.new( name, value )
			end

			cookies
		end
	end
end


# CKLog is a simple logging class.
class CKLog
	class << self
		# Outputs a debug string to STDERR.
		# A backtrace is sent if "exception" is given an error object.
		def debug( string, exception = nil )
			$stderr.write _debug_string(string,exception)
		end

		# Outputs a debug string to a file.
		def debug_to_file( file, string, exception = nil )
			CKFileLock.exclusive_lock( file ) { | out |
				out.write _debug_string(string,exception)
			}
		end

		# Outputs a debug string to "out" needs to be an IO object.
		def debug_to_out( out, string, exception = nil )
			out.write _debug_string(string,exception)
		end

		private

		def _debug_string( string, exception )
			str = "[#{Time.new.localtime}] #{CKApplication.version}: #{string}\n"
			if exception
				str << exception.backtrace.join << "\n"
			end
			str
		end
	end
end


# The super class for locking.
class CKLock
end

# CKFileLock is for locking files.
class CKFileLock < CKLock
	# Creates a shared file lock on a file.
	def self.shared_lock(filename)
		File.open(filename, 'rb') do |io|
			io.flock(File::LOCK_SH)
			yield io
			io.flock(File::LOCK_UN)
		end
	end

	# Creates a exclusive file lock on a file.
	def self.exclusive_lock(filename)
		File.open(filename, 'wb+') do |io|
			io.flock(File::LOCK_EX)
			yield io
			io.flock(File::LOCK_UN)
		end
	end
end


